#include <stdio.h>
#include <string.h>

int __fastcall XorBlock(int shiftFactor, const char *text);
int __fastcall AdjustChecksum(int shiftFactor, int blockSum);
int __fastcall Checksum(const char *text);
void DisplayBinary(const char *text);
void BruteForce();


const char* dictName = "dict.txt";	// Dictionary name
const int maxNumWords = 27000;		// Change this is you enlarge the dictionary
const int maxWordLength = 32;		// Should be more than enough

char word[maxNumWords][maxWordLength];		// Array to hold the words
int wordLen[maxNumWords];					// Cache of word lenths so we don't need to call strlen each time
int numOfLen[maxWordLength];				// Used when quickly changing the last word (see speed optimization)


void main()
{
	char *DanDog  = "Dan's dog could make more ore.";
	char *DanCar  = "Dan's car could hold more people.";
	char *DanBike = "Dan's bike could backfire more often";

	char *DanText = "Dan's truck could crate more stormy.";

	char *text = "Dan's dog";
	const int desired = 0xABE0CC11;

	printf("%X %s\n", Checksum(DanDog), DanDog);
	printf("%X %s\n", Checksum(DanCar), DanCar);
	printf("%X %s\n", Checksum(DanBike), DanBike);

	printf("\n%X %s\n", Checksum(DanText), DanText);

	printf("ABE0CC11 Desired checksum\n\n");

	//printf("10101011111000001100110000010001 (Desired)\n");

	//            "Dan's bike could backfire more often"
	//DisplayBinary("Dan's pppp could pppp more ppppp.");
	//DisplayBinary("Dan's popp could ppop more poppp.");

	BruteForce();
}

void BruteForce()
{
	int count;
	int h = 0, i = 0, j = 0, k = 0;
	int wordNum;
	int offset;
	int len;
	int checksum;
	char buff[64];
	FILE *dict = fopen(dictName, "r");

	for (i = 0; i < 32; i++)
		numOfLen[i] = 0;

	// Read in the dictionary file
	count = 0;
	len = fscanf(dict, "%s", &word[count][0]);
	wordLen[count] = len;
	count++;
	numOfLen[len] = 1;
	while(!feof(dict))
	{
		// Read a word from the file (and store it's length)
		fscanf(dict, "%s", &word[count][0]);
		len = strlen(word[count]);
		if (len != wordLen[count-1])
			numOfLen[len] = 1;
		else
			numOfLen[len]++;
		wordLen[count] = len;
		count++;
	}
	fclose(dict);

	// Quick display of how long this will take
	printf("Number of words in dictionary: %i\n", count);
	printf("(For k, j, i = 1 to count)\n\n");

	// ** Comment out a some for loops if you want to guess a word **
	for (k = 0; k < count; k++)
	{
		for (j = 0; j < count; j++)
		{
			wordNum = 0;
			for (i = 1; i < 32; i++)
			{
				// Change the last word in a faster way...
				if (numOfLen[i] > 0)
				{
					// ** Edit here to guess if you're feeling lucky **
					//len = sprintf(buff, "Dan's truck could %s more %s.", /*word[k],*/ word[j], word[wordNum]);
					len = sprintf(buff, "Dan's %s could %s more %s.", word[k], word[j], word[wordNum]);

					if (len > 36)
						break;

					buff[36] = 0;

					// Speed optimization  (only overwrite last word, instead of the whole string)
					offset = len - i - 1;
					for (h = 0; h < numOfLen[i]; h++)
					{
						// guesses? - computer, truck, lab
						memcpy(&buff[offset], word[wordNum], i);

						checksum = Checksum(buff);
						if (checksum == 0xABE0CC11)
						{
							printf("\nCheat 4\n%s\n", buff);
							printf("(%i, %i, %i) = (%s, %s, %s)\n", k, j, i, word[k], word[j], word[i]);
							return;
						}
						/*
						if (checksum == 0xFF27BC04)
						{
							printf("\nCheat 1\n%s\n", buff);
							printf("(%i, %i, %i)\n", i, j, k);
							//return;
						}
						*/
						wordNum++;
					}
				}
			}
			// Display a status update
			if ((j&0x7F) == 0) 
				printf("j = %i\n", j);
		}
		// Display a status update
		printf("k = %i\n", k);
	}
}

void DisplayBinary(const char *text)
{
	int temp = Checksum(text);
	for (int i = 0; i < 32; i++)
	{
		printf("%i", (temp >> (31-i)) & 1 );
	}
	printf(" %s\n", text);
}

_declspec(naked) int __fastcall XorBlock(int shiftFactor, const char *text)
{
	__asm
	{
		MOV EDX, [EDX]
		MOV EAX, EDX
		ROL EAX, CL
		XOR EAX, EDX
		ROL EAX, CL
		XOR EAX, EDX
		ROL EAX, CL
		XOR EAX, EDX
		RET
	}
}

_declspec(naked) int __fastcall AdjustChecksum(int shiftFactor, int blockSum)
{
	__asm
	{
		SHL ECX, 2
		MOV EAX, EDX
		ROL EAX, CL
		RET
	}
}

_declspec(naked) int __fastcall Checksum(const char *text)
{
	__asm
	{
		PUSH ESI
		PUSH EDI
		PUSH EBX

		LEA ESI, [ECX]		; Load starting address of string
		MOV EDX, [ESI]		; Load first 4 bytes of string (gets repeats)
		XOR EBX, EBX		; Clear Checksum
		MOV EDI, EDX
	LoopDword:
		MOV CL, DL			; Load LSB of block being processed
		MOV CH, 4			; Set loop counter to 4
		SHR CL, 4			; Adjust LSB to get shift factor
		MOV EAX, EDI		; Load DWORD of text (no repeats)
		ROL EAX, 8			; Fixup to make loop mechanics easier
	LoopByte:
		ROL EBX, CL			; Rotate checksum
		XOR EBX, EDX		; Add in current DWORD
		ROR EAX, 8			; Check when NULL terminator is hit
		OR  AL, AL
		JE  LoopEnd

		DEC CH				; Loop 4 times (process each BYTE in DWORD)
		JNZ LoopByte

		XOR BL, AL			; Add BYTE into checksum
		MOV EDX, EDI
		ADD ESI, 4
		MOV EDI, [ESI]
		JMP LoopDword

	LoopEnd:
		MOV EAX, EBX		; Return checksum


		POP EBX
		POP EDI
		POP ESI
		RET
	}
}